@@ -1,23 +1,25 @@
 from __future__ import absolute_import
 
-import datetime
+import datetime, hashlib, logging, time
+from copy import copy
+from xml.etree import ElementTree
+
+from django.utils.datastructures import SortedDict
+from couchdbkit.exceptions import PreconditionFailed
 from couchdbkit.ext.django.schema import *
-import couchforms.const as const
+from couchdbkit.resource import ResourceNotFound
+
 from dimagi.utils.indicators import ComputedDocumentMixin
 from dimagi.utils.parsing import string_to_datetime
 from dimagi.utils.couch.safe_index import safe_index
 from dimagi.utils.couch.database import get_safe_read_kwargs, SafeSaveDocument
-from xml.etree import ElementTree
-from django.utils.datastructures import SortedDict
-from couchdbkit.resource import ResourceNotFound
-import logging
-import hashlib
-from copy import copy
-from couchforms.signals import xform_archived, xform_unarchived
 from dimagi.utils.mixins import UnicodeMixIn
+from dimagi.utils.couch.database import get_db, iter_docs
+
+from couchforms.signals import xform_archived, xform_unarchived
 from couchforms.const import ATTACHMENT_NAME
-from couchdbkit.exceptions import PreconditionFailed
-import time
+from couchforms import const
+
 
 def doc_types():
     """
@@ -55,6 +57,9 @@ class Metadata(DocumentSchema):
             <deviceID />
             <deprecatedID /> 
             <username />
+
+            <!-- CommCare extension -->
+            <appVersion />
         </Meta>
     
     See spec: https://bitbucket.org/javarosa/javarosa/wiki/OpenRosaMetaDataSchema
@@ -68,6 +73,8 @@ class Metadata(DocumentSchema):
     deviceID = StringProperty()
     deprecatedID = StringProperty()
     username = StringProperty()
+    appVersion = StringProperty()
+
 
 class XFormOperation(DocumentSchema):
     """
@@ -79,6 +86,7 @@ class XFormOperation(DocumentSchema):
     date = DateTimeProperty(default=datetime.datetime.utcnow)
     operation = StringProperty() # e.g. "archived", "unarchived"
 
+
 class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
     """An XForms instance."""
     xmlns = StringProperty()
@@ -104,6 +112,31 @@ class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
         extras = get_safe_read_kwargs()
         return db.get(docid, rev=rev, wrapper=cls.wrap, **extras)
 
+    @classmethod
+    def get_forms_by_user(cls, user, start=None, end=None):
+        """
+        Returns a generator object of all forms submitted by user,
+        from start to end dates, if specified.
+        """
+        # Thew couchforms/by_user view returns a zero-indexed month
+        # correct for this by subtracting 1 from month
+        if start is not None:
+            startkey = [user.user_id, start.year, start.month - 1, start.day]
+        else:
+            startkey = [user.user_id]
+        if end is not None:
+            endkey = [user.user_id, end.year, end.month - 1, end.day, {}]
+        else:
+            endkey = [user.user_id, {}]
+        results = cls.view(
+            "couchforms/by_user",
+            startkey=startkey,
+            endkey=endkey,
+            reduce=False
+        ).all()
+        docs = iter_docs(cls.get_db(), [r['id'] for r in results])
+        return (cls(doc) for doc in docs)
+
     @property
     def _form(self):
         return self[const.TAG_FORM]
@@ -131,28 +164,41 @@ class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
                 # couchdbkit chokes on dates that aren't actually dates
                 # so check their validity before passing them up
                 ret = copy(dict(meta_block))
+                for key in ret.keys():
+                    #remove attributes from the meta block
+                    if key.startswith('@'):
+                        del ret[key]
+
+                # couchdbkit erroneously converts appVersion to a Decimal just because it is possible (due to it being within a "dynamic" property)
+                # (see https://github.com/benoitc/couchdbkit/blob/a23343e539370cffcf8b0ce483c712911bb022c1/couchdbkit/schema/properties.py#L1038)
+                if meta_block.get('appVersion') is not None and not isinstance(meta_block['appVersion'], basestring):
+                    if isinstance(meta_block['appVersion'], dict) and '#text' in meta_block['appVersion']:
+                        ret['appVersion'] = str(meta_block['appVersion']['#text'])
+                    else:
+                        ret['appVersion'] = str(meta_block['appVersion'])
+
                 if meta_block:
                     for key in ("timeStart", "timeEnd"):
                         if key in meta_block:
                             if meta_block[key]:
                                 try:
+                                    #try to parse to ensure correctness
                                     parsed = string_to_datetime(meta_block[key])
-                                    ret[key] = parsed
                                 except ValueError:
                                     # we couldn't parse it
                                     del ret[key]
                             else:
                                 # it was empty, also a failure
                                 del ret[key]
-                    # also clean dicts, since those are not allowed
-                    for key in meta_block:
-                        if isinstance(meta_block[key], dict):
+                    # also clean dicts on the return value, since those are not allowed
+                    for key in ret:
+                        if isinstance(ret[key], dict):
                             ret[key] = ", ".join(\
                                 "%s:%s" % (k, v) \
-                                for k, v in meta_block[key].items())
+                                for k, v in ret[key].items())
                 return ret
-            return Metadata(_clean(self._form[const.TAG_META]))
-        
+            return Metadata.wrap(_clean(self.to_json()[const.TAG_FORM][const.TAG_META]))
+
         return None
 
     def __unicode__(self):
@@ -258,6 +304,7 @@ class XFormInstance(SafeSaveDocument, UnicodeMixIn, ComputedDocumentMixin):
         XFormInstance.save(self) # subclasses explicitly set the doc type so force regular save
         xform_unarchived.send(sender="couchforms", xform=self)
 
+
 class XFormError(XFormInstance):
     """
     Instances that have errors go here.
@@ -269,6 +316,7 @@ class XFormError(XFormInstance):
         # XFormInstance we'll force the doc_type to change. 
         self["doc_type"] = "XFormError" 
         super(XFormError, self).save(*args, **kwargs)
+
         
 class XFormDuplicate(XFormError):
     """
@@ -282,6 +330,7 @@ class XFormDuplicate(XFormError):
         # we can't use super because XFormError also sets the doc type
         XFormInstance.save(self, *args, **kwargs)
 
+
 class XFormDeprecated(XFormError):
     """
     After an edit, the old versions go here.
@@ -296,6 +345,7 @@ class XFormDeprecated(XFormError):
         XFormInstance.save(self, *args, **kwargs)
         # should raise a signal saying that this thing got deprecated
 
+
 class XFormArchived(XFormError):
     """
     Archived forms don't show up in reports
@@ -306,6 +356,7 @@ class XFormArchived(XFormError):
         self["doc_type"] = "XFormArchived"
         XFormInstance.save(self, *args, **kwargs)
 
+
 class SubmissionErrorLog(XFormError):
     """
     When a hard submission error (typically bad XML) is received we save it 
